import pytest
import rasterio
from rasterio.crs import CRS
from rasterio.warp import transform_bounds

from usgs_topo_tiler.extent import get_extent

# yapf: disable
# Test cases are generated by a process described here:
# https://github.com/kylebarron/usgs-topo-tiler/issues/8#issuecomment-626104877
AUTOMATED_TEST_CASES = [
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/FL/FL_Fort%20Pierce_346296_1972_10000_geo.tif',
    #     [-80.35, 27.4166667, -80.266667, 27.5], 10000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Comerio_362094_1957_20000_geo.tif',
        [-66.25, 18.125, -66.125, 18.25], 20000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_San%20German_362261_1966_20000_geo.tif',
        [-67.125, 18.0, -67.0, 18.125], 20000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Mayaguez_362154_1955_20000_geo.tif',
        [-67.25, 18.125, -67.125, 18.25], 20000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SC/SC_Capers%20Island_260671_1919_21120_geo.tif',
        [-79.75, 32.75, -79.625, 32.875], 21120, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SC/SC_Charleston_260673_1919_21120_geo.tif',
        [-80.0, 32.75, -79.875, 32.875], 21120, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SC/SC_Sewee%20Bay_260685_1919_21120_geo.tif',
        [-79.75, 32.875, -79.625, 33.0], 21120, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TN/TN_Clouds_149552_1952_24000_geo.tif',
        [-83.75, 36.375, -83.625, 36.5], 24000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AZ/AZ_Mount%20Bigelow_312482_1996_24000_geo.tif',
        [-110.75, 32.375, -110.625, 32.5], 24000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MN/MN_Campbell_504118_1981_24000_geo.tif',
        [-96.5, 46.0, -96.375, 46.125], 24000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SD/SD_Hay%20Canyon%20Butte_344574_1982_25000_geo.tif',
        [-103.125, 43.25, -103.0, 43.375], 25000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/VA/VA_Petersburg_187345_1949_25000_geo.tif',
        [-77.5, 37.125, -77.375, 37.25], 25000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MA/MA_Springfield%20North_463077_1972_25000_geo.tif',
        [-72.625, 42.125, -72.5, 42.25], 25000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Coamo_362385_1946_30000_geo.tif',
        [-66.375, 18.0, -66.25, 18.125], 30000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Cabo%20Rojo_362346_1938_30000_geo.tif',
        [-67.25, 17.875, -67.125, 18.0], 30000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Bayaney_362345_1952_30000_geo.tif',
        [-66.875, 18.25, -66.75, 18.375], 30000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NY/NY_Cleveland_8034260_1944_31680_geo.tif',
        [-76.0, 43.125, -75.875, 43.25], 31680, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Patterson_296401_1916_31680_geo.tif',
        [-121.25, 37.375, -121.125, 37.5], 31680, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/LA/LA_Luling_333934_1935_31680_geo.tif',
        [-90.375, 29.875, -90.25, 30.0], 31680, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Haskell%203-b_128387_1925_48000_geo.tif',
        [-100.0, 33.125, -99.875, 33.25], 48000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Blanco-1_123777_1930_48000_geo.tif',
        [-98.125, 30.375, -98.0, 30.5], 48000, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Breckenridge%201-a_123808_1924_48000_geo.tif',
        [-98.625, 32.875, -98.5, 33.0], 48000, '7.5 X 7.5 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CT/CT_Niantic_461318_1938_62500_geo.tif',
    #     [-72.25, 41.25, -72.125, 41.375], 62500, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/ME/ME_Columbia%20Falls_460327_1921_62500_geo.tif',
        [-67.75, 44.625, -67.625, 44.75], 62500, '7.5 X 7.5 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Moca%20NE_362007_1950_10000_geo.tif',
        [-67.0625, 18.4375, -67.0, 18.5], 10000, '3.75 X 3.75 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Barceloneta%20NE_361929_1950_10000_geo.tif',
        [-66.5625, 18.4375, -66.5, 18.5], 10000, '3.75 X 3.75 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PR/PR_Ciales%20SE_361974_1947_10000_geo.tif',
        [-66.4375, 18.25, -66.375, 18.3125], 10000, '3.75 X 3.75 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Dallas%20(A)_109075_1968_12000_geo.tif',
        [-96.875, 32.8125, -96.8125, 32.875], 12000, '3.75 X 3.75 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/KS/KS_White%20City%20South_122725_1927_20000_geo.tif',
    #     [-96.75, 38.75, -96.5, 39.0], 20000, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/KS/KS_Randolph_801953_1957_20000_geo.tif',
    #     [-96.75, 39.25, -96.5, 39.5], 20000, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/KS/KS_Leonardville_801859_1957_20000_geo.tif',
    #     [-97.0, 39.25, -96.75, 39.5], 20000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/ME/ME_Brooklin_460248_1981_24000_geo.tif',
        [-68.0, 46.25, -67.75, 46.5], 24000, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/PA/PA_Howard_461771_1944_24000_geo.tif',
    #     [-77.75, 41.0, -77.5, 41.25], 24000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/ME/ME_Seven%20Islands_460866_1955_24000_geo.tif',
        [-69.75, 46.75, -69.5, 47.0], 24000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/UT/UT_Wasatch_252663_1922_31680_geo.tif',
        [-111.5, 39.0, -111.25, 39.25], 31680, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MI/MI_Manistique_277856_1931_31680_geo.tif',
    #     [-86.25, 45.75, -86.0, 46.0], 31680, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MN/MN_Juneberry_805494_1930_31680_geo.tif',
        [-96.5, 48.75, -96.25, 49.0], 31680, '15 X 15 Minute'],
    # HTTP 404
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/KY/KY_Higginsport_804206_1928_48000_geo.tif',
    #     [-84.0, 38.75, -83.75, 39.0], 48000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AZ/AZ_Aguila%20Mountains_464992_1927_48000_geo.tif',
        [-113.5, 32.5, -113.25, 32.75], 48000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/ME/ME_Phillips_807620_1929_48000_geo.tif',
        [-70.5, 44.75, -70.25, 45.0], 48000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Cottonwood%20Spring_296650_2003_50000_geo.tif',
        [-116.0, 33.5, -115.75, 33.75], 50000, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Mt%20Michelson%20D-1_463949_1962_50000_geo.tif',
    #     [-144.6, 69.75, -144.0, 70.0], 50000, '15 X 15 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Meade%20River%20D-5_463932_1962_50000_geo.tif',
    #     [-159.0, 70.75, -158.4, 71.0], 50000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AZ/AZ_Benson_314362_1958_62500_geo.tif',
        [-110.5, 31.75, -110.25, 32.0], 62500, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Deep%20Creek_297302_1902_62500_geo.tif',
        [-117.25, 34.25, -117.0, 34.5], 62500, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/OR/OR_Reedsport_282831_1956_62500_geo.tif',
        [-124.25, 43.5, -124.0, 43.75], 62500, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Talkeetna%20D-1_359576_1958_63360_geo.tif',
        [-150.5, 62.75, -150.0, 63.0], 63360, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Port%20Alexander%20A-1_358456_1948_63360_geo.tif',
        [-134.33333330000002, 56.0, -134.0, 56.25], 63360, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Mount%20Michelson%20B-1_357822_1955_63360_geo.tif',
        [-144.6, 69.25, -144.0, 69.5], 63360, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/OK/OK_Norman_802551_1893_96000_geo.tif',
        [-97.5, 35.0, -97.25, 35.25], 96000, '15 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SD/SD_Princes%20Ranch_344584_1980_25000_geo.tif',
        [-101.0, 44.375, -100.75, 44.5], 25000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MA/MA_Hull_351034_1984_25000_geo.tif',
        [-71.0, 42.25, -70.75, 42.375], 25000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MA/MA_Ashfield_350808_1998_25000_geo.tif',
        [-73.0, 42.5, -72.75, 42.625], 25000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MA/MA_Warwick_352419_1932_48000_geo.tif',
        [-72.5, 42.625, -72.25, 42.75], 48000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/MA/MA_Lowell_352418_1931_48000_geo.tif',
        [-71.5, 42.625, -71.25, 42.75], 48000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NY/NY_Starrucca_144032_1924_48000_geo.tif',
        [-75.5, 41.875, -75.25, 42.0], 48000, '7.5 X 15 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/ID/ID_Riggins_239349_1938_96000_geo.tif',
        [-116.5, 45.0, -116.0, 45.5], 96000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Burney_299034_1935_96000_geo.tif',
        [-122.0, 40.5, -121.5, 41.0], 96000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NM/NM_Sulphur%20Spring_189428_1927_96000_geo.tif',
        [-108.75, 36.25, -108.25, 36.75], 96000, '30 X 30 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TN/TN_Lebanon_149953_1942_100000_geo.tif',
    #     [-86.75, 35.75, -86.25, 36.25], 100000, '30 X 30 Minute'],
    # HTTP 404
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NC/NC_Charlotte_473601_1942_100000_geo.tif',
    #     [-81.0, 35.0, -80.5, 35.5], 100000, '30 X 30 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TN/TN_Mc%20Minnville_150199_1943_100000_geo.tif',
    #     [-86.25, 35.25, -85.75, 35.75], 100000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Yosemite_299700_1909_125000_geo.tif',
        [-120.0, 37.5, -119.5, 38.0], 125000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SD/SD_Deadwood_344760_1894_125000_geo.tif',
        [-104.0, 44.0, -103.5, 44.5], 125000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Hayrick_128667_1894_125000_geo.tif',
        [-100.5, 31.5, -100.0, 32.0], 125000, '30 X 30 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Big%20Bar_299708_1911_192000_geo.tif',
    #     [-123.5, 40.5, -123.0, 41.0], 192000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/WA/WA_Mt%20Baker_242611_1909_192000_geo.tif',
        [-122.0, 48.5, -121.5, 49.0], 192000, '30 X 30 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Big%20Bar%20and%20Vicinity_465341_1915_250000_geo.tif',
    #     [-123.75, 40.5, -123.0, 41.0], 250000, '30 X 30 Minute'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Big%20Bar%20and%20Vicinity_299743_1915_250000_geo.tif',
    #     [-123.75, 40.5, -123.0, 41.0], 250000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/WA/WA_Mt%20Baker%20Dist_242737_1915_250000_geo.tif',
        [-122.0, 48.5, -121.5, 49.0], 250000, '30 X 30 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NV/NV_Pahute%20Mesa_321545_1979_100000_geo.tif',
        [-117.0, 37.0, -116.0, 37.5], 100000, '30 X 60 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/SD/SD_Timber%20Lake_344719_1986_100000_geo.tif',
        [-102.0, 45.0, -101.0, 45.5], 100000, '30 X 60 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/IL/IL_Chicago_310155_1980_100000_geo.tif',
        [-88.0, 41.5, -87.0, 42.0], 100000, '30 X 60 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Cotulla_123888_1929_125000_geo.tif',
        [-100.0, 28.0, -99.0, 28.5], 125000, '30 X 60 Minute'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AL/AL_Phenix%20City_305931_1958_250000_geo.tif',
        [-86.0, 32.0, -84.0, 33.0], 250000, '1 X 2 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/TX/TX_Wichita%20Falls_117118_1954_250000_geo.tif',
        [-100.0, 33.0, -98.0, 34.0], 250000, '1 X 2 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/OR/OR_Roseburg_283349_1958_250000_geo.tif',
        [-124.0, 43.0, -122.0, 44.0], 250000, '1 X 2 Degree'],
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Demarcation%20Point_463682_1964_250000_geo.tif',
    #     [-144.0, 69.0, -141.0, 70.0], 250000, '1 X 3 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Chandler%20Lake_360740_1956_250000_geo.tif',
        [-153.0, 68.0, -150.0, 69.0], 250000, '1 X 3 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Seward_361838_1953_250000_geo.tif',
        [-150.0, 60.0, -147.0, 61.0], 250000, '1 X 3 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NV/NV_Saint%20Thomas_321775_1886_250000_geo.tif',
        [-115.0, 36.0, -114.0, 37.0], 250000, '1 X 1 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/NV/NV_Roberts%20Mountains_321771_1929_250000_geo.tif',
        [-117.0, 39.0, -116.0, 40.0], 250000, '1 X 1 Degree'],
    # HTTP 404
    # [
    #     'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AZ/AZ_Fort%20Defiance_315488_1886_250000_geo.tif',
    #     [-110.0, 35.0, -109.0, 36.0], 250000, '1 X 1 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Wainwright_361660_1955_250000_geo.tif',
        [-163.0, 70.0, -159.0, 71.0], 250000, '1 X 4 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Teller_361551_1950_250000_geo.tif',
        [-169.0, 65.0, -165.0, 66.0], 250000, '1 X 4 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Teller_361554_1950_250000_geo.tif',
        [-169.0, 65.0, -165.0, 66.0], 250000, '1 X 4 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/VA/VA_Eastville_189247_1966_250000_geo.tif',
        [-76.0, 36.0, -75.0, 38.0], 250000, '2 X 1 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/WA/WA_Copalis%20Beach_240663_1957_250000_geo.tif',
        [-125.0, 46.0, -124.0, 48.0], 250000, '2 X 1 Degree'],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/CA/CA_Eureka_465608_1949_250000_geo.tif',
        [-125.0, 40.0, -124.0, 42.0], 250000, '2 X 1 Degree']]
# yapf: enable

CUSTOM_TEST_CASES = [
    # Scale == 63360
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AZ/AZ_Wagoner_464904_1948_63360_geo.tif',
        [-112.75, 34, -112.5, 34.25], 63360, ''],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Sitka%20D-5_359166_1948_63360_geo.tif',
        [-135.66666669999998, 57.75, -135.33333330000002, 58], 63360, ''],
    # Tests right on boundary
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Mount%20Katmai%20D-4_357744_1951_63360_geo.tif',
        [-155.33333330000002, 58.75, -155, 59], 63360, ''],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Iliamna%20A-6_356232_1951_63360_geo.tif',
        [-155.25, 59, -154.875, 59.25], 63360, ''],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Tyonek%20A-1_359987_1951_63360_geo.tif',
        [-150.375, 61, -150, 61.25], 63360, ''],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Medfra%20A-4_357441_1959_63360_geo.tif',
        [-155, 63, -154.5, 63.25], 63360, ''],
    [
        'https://prd-tnm.s3.amazonaws.com/StagedProducts/Maps/HistoricalTopo/GeoTIFF/AK/AK_Lookout%20Ridge%20B-1_357171_1985_63360_geo.tif',
        [-156.6, 69.25, -156, 69.5], 63360, '']]


@pytest.mark.parametrize(
    "url,map_bounds,scale,grid_size",
    [*CUSTOM_TEST_CASES, *AUTOMATED_TEST_CASES])
def test_extent(url, map_bounds, scale, grid_size):
    with rasterio.open(url) as r:
        # Convert image bounds to wgs84
        image_wgs_bounds = transform_bounds(
            r.crs, CRS.from_epsg(4326), *r.bounds)

        extent = get_extent(image_wgs_bounds, url)
        for x, y in zip(extent, map_bounds):
            msg = f'{extent}, {map_bounds} not approx equal'
            assert x == pytest.approx(y), msg
